package org.jp.illg.dstar.model;

import java.util.Arrays;

import org.jp.illg.dstar.DStarDefines;
import org.jp.illg.dstar.model.defines.RepeaterFlags;
import org.jp.illg.dstar.model.defines.RepeaterRoute;
import org.jp.illg.dstar.util.DStarUtils;
import org.jp.illg.util.ArrayUtil;

import lombok.Getter;
import lombok.NonNull;

public class Header implements Cloneable{

	public static final byte FLAG_REPEATER_MASK = (byte)0x7;
/*
	public static final byte FLAG_REPEATER_CONTROL= (byte)0x7;
	public static final byte FLAG_AUTO_REPLY = (byte)0x6;
	public static final byte FLAG_REQUEST_RETRY = (byte)0x4;
	public static final byte FLAG_ACK = (byte)0x3;
	public static final byte FLAG_NO_REPLY = (byte)0x2;
	public static final byte FLAG_CANT_REPEAT = (byte)0x1;
*/

	/**
	 * Flags
	 */
	private byte[] flags = new byte[3];

	/**
	 * Repeater 2 Callsign
	 */
	private char[] repeater2Callsign = new char[DStarDefines.CallsignFullLength];

	/**
	 * Repeater 1 Callsign
	 */
	private char[] repeater1Callsign = new char[DStarDefines.CallsignFullLength];

	/**
	 * Your Callsign
	 */
	private char[] yourCallsign = new char[DStarDefines.CallsignFullLength];

	/**
	 * My Callsign
	 */
	private char[] myCallsign = new char[DStarDefines.CallsignFullLength];

	/**
	 * My Callsign Additional
	 */
	private char[] myCallsignAdd = new char[DStarDefines.CallsignShortLength];

	/**
	 * CRC
	 */
	private byte[] crc = new byte[2];

	/**
	 * Source Repeater 1 Callsign
	 */
	@Getter
	private char[] sourceRepeater1Callsign = new char[DStarDefines.CallsignFullLength];

	/**
	 * Source Repeater 2 Callsign
	 */
	@Getter
	private char[] sourceRepeater2Callsign = new char[DStarDefines.CallsignFullLength];


	public Header() {
		super();

		clear();
	}

	public Header(
		final char[] yourCallsign,
		final char[] repeater1Callsign, final char[] repeater2Callsign,
		final char[] myCallsign, final char[] myCallsignAdd
	) {
		this();

		setYourCallsign(yourCallsign);
		setRepeater1Callsign(repeater1Callsign);
		setRepeater2Callsign(repeater2Callsign);
		setMyCallsign(myCallsign);
		setMyCallsignAdd(myCallsignAdd);

	}

	@Override
	public Header clone() {
		Header copy = null;
		try {
			copy = (Header)super.clone();

			copy.flags = this.flags.clone();
			copy.repeater2Callsign = this.repeater2Callsign.clone();
			copy.repeater1Callsign = this.repeater1Callsign.clone();
			copy.yourCallsign = this.yourCallsign.clone();
			copy.myCallsign = this.myCallsign.clone();
			copy.myCallsignAdd = this.myCallsignAdd.clone();
			copy.crc = this.crc.clone();
			copy.sourceRepeater1Callsign = this.sourceRepeater1Callsign.clone();
			copy.sourceRepeater2Callsign = this.sourceRepeater2Callsign.clone();

		}catch(CloneNotSupportedException ex) {
			throw new RuntimeException(ex);
		}

		return copy;
	}

	public void clear() {
		Arrays.fill(this.getFlags(), (byte)0x00);
		Arrays.fill(this.getRepeater2Callsign(), ' ');
		Arrays.fill(this.getRepeater1Callsign(), ' ');
		Arrays.fill(this.getYourCallsign(), ' ');
		Arrays.fill(this.getMyCallsign(), ' ');
		Arrays.fill(this.getMyCallsignAdd(), ' ');
		Arrays.fill(this.getCrc(), (byte)0x00);
		Arrays.fill(this.getSourceRepeater1Callsign(), ' ');
		Arrays.fill(this.getSourceRepeater2Callsign(), ' ');
	}

	/**
	 * @return flags
	 */
	public byte[] getFlags() {
		return flags;
	}

	/**
	 * @param flags セットする flags
	 */
	public void setFlags(byte[] flags) {
		if(flags == null || flags.length <= 0) {return;}

		ArrayUtil.copyOf(this.flags, flags);
	}

	public void setRepeaterFlag(RepeaterFlags flag) {
		if(flag == null) {return;}

		getFlags()[0] = (byte)((getFlags()[0] & ~FLAG_REPEATER_MASK) | flag.getValue());
	}

	public boolean isSetRepeaterFlag(@NonNull RepeaterFlags flag) {

		return (getFlags()[0] & RepeaterFlags.getMask()) == flag.getValue();
	}

	public void setRepeaterRouteFlag(RepeaterRoute route) {
		if(route == null) {return;}

		getFlags()[0] = (byte)((getFlags()[0] & ~(RepeaterRoute.getMask())) | route.getValue());
	}

	public boolean isSetRepeaterRouteFlag(@NonNull RepeaterRoute route) {

		return (getFlags()[0] & RepeaterRoute.getMask()) == route.getValue();
	}


	public char[] getRepeater2Callsign() {
		return repeater2Callsign;
	}

	public void setRepeater2Callsign(char[] callsign) {
		if(callsign == null || callsign.length <= 0) {return;}

		ArrayUtil.copyOf(this.repeater2Callsign, callsign);
	}

	public void setRepeater2Callsign(String callsign) {
		if(callsign == null || callsign.length() <= 0) {return;}

		setRepeater2Callsign(callsign.toCharArray());
	}


	public char[] getRepeater1Callsign() {
		return repeater1Callsign;
	}

	public void setRepeater1Callsign(char[] callsign) {
		if(callsign == null || callsign.length <= 0) {return;}

		ArrayUtil.copyOf(this.repeater1Callsign, callsign);
	}

	public void setRepeater1Callsign(String callsign) {
		if(callsign == null || callsign.length() <= 0) {return;}

		setRepeater1Callsign(callsign.toCharArray());
	}


	public char[] getYourCallsign() {
		return yourCallsign;
	}

	public void setYourCallsign(char[] callsign) {
		if(callsign == null || callsign.length <= 0) {return;}

		ArrayUtil.copyOf(this.yourCallsign, callsign);
	}

	public void setYourCallsign(String callsign) {
		if(callsign == null || callsign.length() <= 0) {return;}

		setYourCallsign(callsign.toCharArray());
	}


	public char[] getMyCallsign() {
		return myCallsign;
	}

	public void setMyCallsign(char[] callsign) {
		if(callsign == null || callsign.length <= 0) {return;}

		ArrayUtil.copyOf(this.myCallsign, callsign);
	}

	public void setMyCallsign(String callsign) {
		if(callsign == null || callsign.length() <= 0) {return;}

		setMyCallsign(callsign.toCharArray());
	}


	public char[] getMyCallsignAdd() {
		return myCallsignAdd;
	}

	public void setMyCallsignAdd(char[] callsign) {
		if(callsign == null || callsign.length <= 0) {return;}

		ArrayUtil.copyOf(this.myCallsignAdd, callsign);
	}

	public void setMyCallsignAdd(String callsign) {
		if(callsign == null || callsign.length() <= 0) {return;}

		setMyCallsignAdd(callsign.toCharArray());
	}


	public byte[] getCrc() {
		return crc;
	}

	public int getCrcInt() {
		return ((crc[1] << 8) & 0xFF00) | crc[0];
	}

	public void setCrc(final byte[] crc) {
		if(crc == null || crc.length <= 0) {return;}

		ArrayUtil.copyOf(this.crc, crc);
	}

	public void setCrcInt(final int crc) {
		getCrc()[0] = (byte)(crc & 0xFF);
		getCrc()[1] = (byte)((crc >> 8) & 0xFF);
	}


	public void setSourceRepeater1Callsign(char[] callsign) {
		if(callsign == null || callsign.length <= 0) {return;}

		ArrayUtil.copyOf(this.sourceRepeater1Callsign, callsign);
	}

	public void setSourceRepeater1Callsign(String callsign) {
		if(callsign == null || callsign.length() <= 0) {return;}

		setSourceRepeater1Callsign(callsign.toCharArray());
	}


	public void setSourceRepeater2Callsign(char[] callsign) {
		if(callsign == null || callsign.length <= 0) {return;}

		ArrayUtil.copyOf(this.sourceRepeater2Callsign, callsign);
	}

	public void setSourceRepeater2Callsign(String callsign) {
		if(callsign == null || callsign.length() <= 0) {return;}

		setSourceRepeater2Callsign(callsign.toCharArray());
	}

	public void saveRepeaterCallsign() {
		setSourceRepeater1Callsign(getRepeater1Callsign());
		setSourceRepeater2Callsign(getRepeater2Callsign());
	}

	public void reformatCallsigns() {
		reformatCallsign(getMyCallsign());
		reformatCallsign(getMyCallsignAdd());
		reformatCallsign(getYourCallsign());
		reformatCallsign(getRepeater1Callsign());
		reformatCallsign(getRepeater2Callsign());
	}

	public int calcCRC() {
		final byte[] data = new byte[39];
		ArrayUtil.copyOfRange(data, 0, getFlags());
		ArrayUtil.copyOfRange(data, 3, getRepeater2Callsign());
		ArrayUtil.copyOfRange(data, 11, getRepeater1Callsign());
		ArrayUtil.copyOfRange(data, 19, getYourCallsign());
		ArrayUtil.copyOfRange(data, 27, getMyCallsign());
		ArrayUtil.copyOfRange(data, 35, getMyCallsignAdd());

		final int crc = DStarUtils.calcCRC(data, data.length);

		setCrcInt(crc);

		return crc;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(flags);
		result = prime * result + Arrays.hashCode(myCallsign);
		result = prime * result + Arrays.hashCode(myCallsignAdd);
		result = prime * result + Arrays.hashCode(repeater1Callsign);
		result = prime * result + Arrays.hashCode(repeater2Callsign);
		result = prime * result + Arrays.hashCode(yourCallsign);
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Header other = (Header) obj;
		if (!Arrays.equals(flags, other.flags))
			return false;
		if (!Arrays.equals(myCallsign, other.myCallsign))
			return false;
		if (!Arrays.equals(myCallsignAdd, other.myCallsignAdd))
			return false;
		if (!Arrays.equals(repeater1Callsign, other.repeater1Callsign))
			return false;
		if (!Arrays.equals(repeater2Callsign, other.repeater2Callsign))
			return false;
		if (!Arrays.equals(yourCallsign, other.yourCallsign))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return toString(0);
	}

	public String toString(int indent) {
		if(indent < 0) {indent = 0;}

		StringBuffer sb = new StringBuffer();

		for(int count = 0; count < indent; count++)
			sb.append(' ');

		sb.append("[");
		sb.append(this.getClass().getSimpleName());
		sb.append("]:");

		sb.append("Flags=");
		sb.append(String.format("0x%02X 0x%02X 0x%02X", getFlags()[0], getFlags()[1], getFlags()[2]));

		sb.append("/");
		sb.append("MY=");
		sb.append(getMyCallsign());
		sb.append("_");
		sb.append(getMyCallsignAdd());

		sb.append("/");
		sb.append("UR=");
		sb.append(getYourCallsign());

		sb.append("/");
		sb.append("RPT1=");
		sb.append(getRepeater1Callsign());
		sb.append('(');
		sb.append(getSourceRepeater1Callsign());
		sb.append(')');

		sb.append("/");
		sb.append("RPT2=");
		sb.append(getRepeater2Callsign());
		sb.append('(');
		sb.append(getSourceRepeater2Callsign());
		sb.append(')');

		return sb.toString();
	}

	private void reformatCallsign(@NonNull char[] callsign) {
		final String callsignStr = String.valueOf(callsign);

		final String formatedCallsign =
			DStarUtils.reformatCallsign(callsignStr);

		ArrayUtil.copyOf(callsign, formatedCallsign.toCharArray());
	}
}
